@@ -18,7 +18,8 @@ from pathlib import Path
 
 from appdirs import AppDirs
 import PySide6
-from PySide6.QtCore import QModelIndex, QPoint, QItemSelectionModel, QTimer, Signal, Slot, QItemSelection, QObject
+from PySide6.QtCore import QModelIndex, QPoint, QItemSelectionModel, QTimer, Signal, Slot, QItemSelection, QObject, \
+    QSize
 from PySide6.QtGui import QCloseEvent, QPixmap, QColor, Qt, QImage, QIcon, QCursor, QAction
 from PySide6.QtWidgets import QMainWindow, QApplication, QHBoxLayout, QSizePolicy, QMessageBox, QMenu, \
     QVBoxLayout, QLabel, QDockWidget, QListView, QWidget, QDialogButtonBox, QAbstractItemView
@@ -30,7 +31,8 @@ from maphis.common.label_change import generate_change_command
 from maphis.common.label_hierarchy import LabelHierarchy
 from maphis.common.local_storage import Storage, LocalStorage
 from maphis.common.new_plugin import create_new_plugin, interpolate_class_doc
-from maphis.common.photo import LabelImg, Photo
+from maphis.common.photo import LabelImg, Photo, PhotoUpdate, PhotoUpdateType, UpdateEvent, UpdateContext, \
+    LabelImageUpdate, LabelImageUpdateType
 from maphis.common.plugin import RegionComputation, GeneralAction, ActionContext
 from maphis.common.scale_setting_widget import ScaleSettingWidget, ScaleItemDelegate
 from maphis.common.state import State
@@ -40,7 +42,8 @@ from maphis.common.user_params import UserParamWidgetBinding, \
     create_params_widget_with_buttons
 from maphis.common.utils import choose_folder
 from maphis.image_list_delegate import ImageListDelegate, ImageListView
-from maphis.image_list_model import ImageListModel, ImageListSortFilterProxyModel, ROLE_IMAGE_NAME, ROLE_IMPORT_TIME
+from maphis.image_list_model import ImageListModel, ImageListSortFilterProxyModel, ROLE_IMAGE_NAME, ROLE_IMPORT_TIME, \
+    ROLE_PHOTO
 from maphis.image_viewer import ImageViewer
 from maphis.import_dialog import ImportDialog
 from maphis.import_utils import TempStorage
@@ -151,7 +154,7 @@ class MAPHIS(QMainWindow):
 
         self.image_list_model = ImageListModel()
 
-        self.image_list_proxy_model = ImageListSortFilterProxyModel(self.state)
+        self.image_list_proxy_model = ImageListSortFilterProxyModel()
         self.image_list_proxy_model.setSourceModel(self.image_list_model)
         self.image_list_proxy_model.setSortCaseSensitivity(Qt.CaseSensitivity.CaseInsensitive)
         self.image_list_proxy_model.setSortRole(Qt.ItemDataRole.DisplayRole)
@@ -206,6 +209,12 @@ class MAPHIS(QMainWindow):
         # QObject.connect(QApplication.instance(), PySide6.QtCore.SIGNAL("focusChanged(QWidget *, QWidget *)"), self.handle_focus_changed)
         QObject.connect(QApplication.instance(), PySide6.QtCore.SIGNAL("focusChanged(QWidget *, QWidget *)"), self.handle_focus_changed)
 
+        screen = QApplication.primaryScreen()
+        self.resize(screen.availableSize())
+
+        with importlib.resources.path('maphis.resources', 'caution.png') as icon_path:
+            self._caution_icon: QIcon = QIcon(str(icon_path))
+
     def handle_focus_changed(self, old: QWidget, new: QWidget):
         # print("calling app.py:focus changed")
         # if old == self.image_list:
@@ -219,14 +228,14 @@ class MAPHIS(QMainWindow):
         pass
 
     def _setup_scale_setting_widget(self):
-        self.scale_setting_widget: ScaleSettingWidget = ScaleSettingWidget(self.state)
+        self.scale_setting_widget: ScaleSettingWidget = ScaleSettingWidget(State())
         self.scale_setting_widget.accepted.connect(self._handle_scales_accepted)
         self.scale_setting_widget.cancelled.connect(self.switch_to_label_editor)
 
-        self.scale_setting_widget.image_viewer.first_photo_requested.connect(self.fetch_first_photo)
-        self.scale_setting_widget.image_viewer.last_photo_requested.connect(self.fetch_last_photo)
-        self.scale_setting_widget.image_viewer.next_photo_requested.connect(self.fetch_next_photo)
-        self.scale_setting_widget.image_viewer.prev_photo_requested.connect(self.fetch_prev_photo)
+        # self.scale_setting_widget.image_viewer.first_photo_requested.connect(self.fetch_first_photo)
+        # self.scale_setting_widget.image_viewer.last_photo_requested.connect(self.fetch_last_photo)
+        # self.scale_setting_widget.image_viewer.next_photo_requested.connect(self.fetch_next_photo)
+        # self.scale_setting_widget.image_viewer.prev_photo_requested.connect(self.fetch_prev_photo)
 
     def _setup_plugins_menu_entry(self):
         self.plugins_menu = QMenu('Plugins')
@@ -280,12 +289,16 @@ class MAPHIS(QMainWindow):
         widg = QWidget()
         widg.setLayout(lay)
 
+        screen = QApplication.primaryScreen()
+        avail_size = screen.availableSize()
+
         self.dw_image_list.setWidget(widg)
         self.dw_image_list.setWindowTitle("Photos")
         self.dw_image_list.setFloating(False)
         self.dw_image_list.setFeatures(QDockWidget.DockWidgetFloatable | QDockWidget.DockWidgetMovable)
         self.dw_image_list.setSizePolicy(QSizePolicy.Preferred, QSizePolicy.Expanding)
         self.addDockWidget(Qt.LeftDockWidgetArea, self.dw_image_list)
+        self.dw_image_list.setMaximumWidth(int(0.15 * avail_size.width()))
 
         # Move this block to change the placement of this dockwidget
         self.dw_segmentation = QDockWidget()
@@ -295,6 +308,7 @@ class MAPHIS(QMainWindow):
         self.addDockWidget(Qt.RightDockWidgetArea, self.dw_segmentation)
         self.dw_segmentation.setSizePolicy(QSizePolicy.Preferred, QSizePolicy.Preferred)
         self.dw_segmentation.sizePolicy().setVerticalStretch(1)
+        self.dw_segmentation.setMaximumWidth(int(0.15 * avail_size.width()))
         # end of block
 
         self.dw_toolbox = QDockWidget()
@@ -304,6 +318,7 @@ class MAPHIS(QMainWindow):
         self.addDockWidget(Qt.RightDockWidgetArea, self.dw_toolbox)
         self.dw_toolbox.setSizePolicy(QSizePolicy.Preferred, QSizePolicy.Preferred)
         self.dw_toolbox.sizePolicy().setVerticalStretch(2)
+        self.dw_toolbox.setMaximumWidth(int(0.15 * avail_size.width()))
 
         self.dw_labels = QDockWidget()
         self.dw_labels.setWindowTitle("Labels")
@@ -312,6 +327,7 @@ class MAPHIS(QMainWindow):
         self.addDockWidget(Qt.RightDockWidgetArea, self.dw_labels)
         self.dw_labels.setSizePolicy(QSizePolicy.Preferred, QSizePolicy.Preferred)
         self.dw_labels.sizePolicy().setVerticalStretch(4)
+        self.dw_labels.setMaximumWidth(int(0.15 * avail_size.width()))
 
         self.dw_image_list.setEnabled(False)
         self.dw_labels.setEnabled(False)
@@ -355,6 +371,7 @@ class MAPHIS(QMainWindow):
         self.label_editor.image_viewer.stop_sequence_requested.connect(self.stop_sequence)
 
         self.label_editor.unsaved_changes.connect(self.enable_actionSave)
+        self.label_editor.widget.setMaximumWidth(int(0.7 * QApplication.primaryScreen().availableSize().width()))
 
     def play_sequence(self):
         if self.playback_state.timer is not None:
@@ -523,8 +540,12 @@ class MAPHIS(QMainWindow):
             self.image_list_proxy_model.setSortRole(ROLE_IMPORT_TIME)
             self.image_list_proxy_model.sort(0, Qt.SortOrder.DescendingOrder)
         self.image_list_proxy_model.invalidate()
-        sel_index = self.image_list.selectedIndexes()[0]
+        selection = self.image_list.selectedIndexes()
+        if len(selection) == 0:
+            return
+        sel_index = selection[0]
         index = self.image_list_proxy_model.mapToSource(sel_index)
+        self.measurements_viewer.update_measurements_view()
         # print(f'these are selection {[idx.row() for idx in sel_index]}')
         self.fetch_photo(index.row())
 
@@ -662,7 +683,7 @@ class MAPHIS(QMainWindow):
         for module in modules:
             members = inspect.getmembers(module)
             for obj_name, obj in members:
-                if not obj_name.startswith('Tool_'):
+                if not inspect.isclass(obj) or not issubclass(obj, Tool) or obj == Tool:
                     continue
                 if inspect.isclass(obj) and not inspect.isabstract(obj):
                     tool = obj(self.state)
@@ -674,7 +695,8 @@ class MAPHIS(QMainWindow):
         logger.info(f'loaded {len(tools)} tools')
 
     def handle_tags_filter_changed(self, active_tags: typing.List[str]):
-        self.image_list_proxy_model.invalidate()
+        # self.image_list_proxy_model.invalidate()
+        self.image_list_proxy_model.set_filter_tags(active_tags)
         self.fetch_first_photo()
 
     def handle_tag_filter_changed(self, filter_index: int):
@@ -783,11 +805,12 @@ class MAPHIS(QMainWindow):
                 hier_copy.name = lbl_name
                 storage.set_label_hierarchy(lbl_name, hier_copy)
         self.plugins_menu.setEnabled(True)
+        old_storage = self.state.storage
         self.storage = storage
         self.storage.storage_update.connect(self.handle_storage_updated)
+        self.storage.update_photo.connect(self.handle_update_photo)
+        self.storage.all_labels_properties_up_to_date.connect(self.handle_all_labels_properties_up_to_date)
         self.state.storage = storage
-        self.tag_filter_widget.tag_chooser.clear_tags()
-        self.state.active_tags_filter = []
         self.action_context.storage = self.state.storage
         self.state.current_label_name = self.storage.default_label_image
         self.state.set_label_constraint(self.state.current_label_name)
@@ -799,6 +822,14 @@ class MAPHIS(QMainWindow):
 
         self.image_list_model.initialize(self.storage.image_paths, self.thumbnail_storage, 0, self.state.storage)
 
+        self.state.image_list_model = self.image_list_proxy_model
+        self.scale_setting_widget.state.image_list_model = self.image_list_proxy_model
+
+        self.measurements_viewer.set_new_storage(self.state.storage, old_storage)
+
+        self.tag_filter_widget.tag_chooser.clear_tags()
+        self.state.active_tags_filter = []
+
         self.current_idx = self.image_list_proxy_model.index(0, 0)
         self.label_editor.widget.setEnabled(True)
         self.measurements_viewer.setEnabled(True)
@@ -813,7 +844,9 @@ class MAPHIS(QMainWindow):
 
         self.scale_setting_widget.initialize(self.state.storage)
 
+        self.dw_image_list.setEnabled(True)
         self.image_list.selectionModel().setCurrentIndex(self.current_idx, QItemSelectionModel.Select)
+        self.handle_current_changed(self.current_idx, QModelIndex())
 
         if self.state.current_photo is not None:
             self.setWindowTitle(f'MAPHIS - ({self.state.storage.location}) - {self.state.current_photo.image_name}')
@@ -824,7 +857,6 @@ class MAPHIS(QMainWindow):
         self.ui.actionOpen_project_folder.setEnabled(True)
         self.state.constraint_label = 0
 
-        self.dw_image_list.setEnabled(True)
         self.dw_labels.setEnabled(True)
         self.dw_toolbox.setEnabled(True)
         self.dw_segmentation.setEnabled(True)
@@ -836,6 +868,10 @@ class MAPHIS(QMainWindow):
         self.image_list.setMaximumWidth(self.thumbnail_storage.thumbnail_size[0] + self.image_list.verticalScrollBar().rect().width() + 2 * self.image_list.frameWidth())
         self.image_list.setSizePolicy(QSizePolicy.Minimum, QSizePolicy.Expanding)
 
+        for photo in self.storage.images:
+            if not photo['Labels'].has_valid_measurements():
+                self.put_caution_icon_on_measurements_tab(True)
+
     # def handle_storage_updated(self, data: typing.Dict[str, typing.Any]):
     def handle_storage_updated(self, update: StorageUpdate):
         # if 'tags' in data:
@@ -855,6 +891,24 @@ class MAPHIS(QMainWindow):
     #         idx = all_tags.index(tag)
     #         self.ui.cmbTags
 
+    def handle_update_photo(self, event: UpdateEvent):
+        if event.update_context == UpdateContext.LabelImg:
+            update_obj: LabelImageUpdate = event.update_obj
+            if update_obj.update_type == LabelImageUpdateType.PropertiesInvalid:
+                self.put_caution_icon_on_measurements_tab(True)
+
+    def handle_all_labels_properties_up_to_date(self, label_img_name: str):
+        if label_img_name == 'Labels':
+            self.put_caution_icon_on_measurements_tab(False)
+
+    def put_caution_icon_on_measurements_tab(self, caution: bool):
+        tabbar = self.ui.tabWidget.tabBar()
+        if caution:
+            tabbar.setIconSize(QSize(16, 16))
+            tabbar.setTabIcon(1, self._caution_icon)
+        else:
+            tabbar.setTabIcon(1, QIcon())
+
     def fetch_photo(self, idx: int):
         if idx < 0 or idx >= self.image_list_model.rowCount():
             return
@@ -1008,6 +1062,7 @@ class MAPHIS(QMainWindow):
                 logger.info('Stopping ThumbnailStorage')
                 # self.thumbnail_storage.stop()
             self.state.reset_state()
+            self.scale_setting_widget.state.reset_state()
             logger.info(f'Attemtpting to load LocalStorage from {folder}')
             strg = LocalStorage.load_from(folder)  # <-- This is where the exception happens if the .json is not found.
 
@@ -1053,6 +1108,7 @@ class MAPHIS(QMainWindow):
             #       What happens when creating a project with a name/location that already exists?
             self.import_dialog.hide()
             self.update_applyToUnsegmented_state()
+            self.label_editor.adjust_level_approval_switch_width()
         except IOError as e:
             error_message = QMessageBox(self)
             error_message.setIcon(QMessageBox.Warning)
@@ -1074,6 +1130,8 @@ class MAPHIS(QMainWindow):
         # self.thumbnail_storage.initialize(self.state.storage)
         self.image_list_model.initialize(self.state.storage.image_paths, self.thumbnail_storage, 0,
                                          self.state.storage)
+        self.measurements_viewer.set_new_storage(self.state.storage, self.state.storage)
+        # self.measurements_viewer.model.update_model()
         self.import_dialog.hide()
         self.command_executor.update()
         if self.state.storage.image_count == 1:
@@ -1205,12 +1263,17 @@ class MAPHIS(QMainWindow):
         self.image_list.setSelectionMode(QAbstractItemView.SingleSelection)
 
         self.switch_to_scale_setting()
-        if photo.image_name != self.state.current_photo.image_name:
-            idx = self.state.storage.image_names.index(photo.image_name)
-            index = self.image_list_proxy_model.mapFromSource(self.image_list_model.index(idx, 0))
-            self.image_list.setCurrentIndex(index)
-        else:
-            self.scale_setting_widget.image_viewer.set_photo(self.state.current_photo, True)
+        idx = self.image_list_proxy_model.image_paths.index(photo.image_path)
+        index = self.image_list_proxy_model.index(idx, 0, QModelIndex())
+        self.scale_setting_widget.image_viewer.set_photo(index.data(ROLE_PHOTO))
+        # if photo.image_name != self.state.current_photo.image_name:
+        #     # idx = self.state.storage.image_names.index(photo.image_name)
+        #     idx = self.image_list_proxy_model.image_paths.index(photo.image_path)
+        #     index = self.image_list_proxy_model.index(idx, 0, QModelIndex())
+            # index = self.image_list_proxy_model.mapFromSource(self.image_list_model.index(idx, 0))
+            # self.image_list.setCurrentIndex(index)
+        # else:
+        # self.scale_setting_widget.image_viewer.set_photo(self.state.current_photo, True)
 
     def handle_delete_photo_requested(self, photo: Photo):
         # Ask for confirmation (warn explicitly if the action cannot be undone), and if confirmed, delete the photo from the project.
@@ -1310,7 +1373,7 @@ class MAPHIS(QMainWindow):
         return execute
 
     def compute_regions3(self, reg_comp: RegionComputation, process_mode: ProcessType):
-        if not reg_comp.initialize() and not (can_execute := reg_comp.can_be_executed)[0]:
+        if not (can_execute := reg_comp.can_be_executed)[0]:
             QMessageBox.information(self, f'Cannot run {reg_comp.info.name}', can_execute[1])
             return
         if process_mode == ProcessType.ALL_PHOTOS:
@@ -1326,6 +1389,9 @@ class MAPHIS(QMainWindow):
         block_op = BlockingOperation(self.state.storage, img_idxs,
                                      self._region_comp_operation(reg_comp),
                                      self._process_region_operation_result, parent=self)
+        if not (init_result := reg_comp.initialize())[0]:
+            QMessageBox.information(self, f'Cannot run {reg_comp.info.name}', init_result[1])
+            return
         block_op.start()
         self.state.current_photo = self.state.storage.get_photo_by_name(self.state.current_photo.image_name)
         self.update_applyToUnsegmented_state()
@@ -1341,7 +1407,24 @@ class MAPHIS(QMainWindow):
 
         self.scale_setting_widget.showMaximized()
 
-        self.current_image_viewer = self.scale_setting_widget.image_viewer  # TODO unify image viewer and change only the context around it
+        # self.scale_setting_widget.state = State()
+
+        proxy_model = ImageListSortFilterProxyModel()
+        proxy_model.setSourceModel(self.image_list_model)
+        proxy_model.setSortRole(self.image_list_proxy_model.sortRole())
+        proxy_model.setSortCaseSensitivity(Qt.CaseSensitivity.CaseInsensitive)
+        proxy_model.setDynamicSortFilter(True)
+
+        if proxy_model.sortRole() == ROLE_IMPORT_TIME:
+            proxy_model.sort(0, Qt.SortOrder.DescendingOrder)
+        else:
+            proxy_model.sort(0, Qt.SortOrder.AscendingOrder)
+
+        self.scale_setting_widget.state.storage = self.state.storage
+        self.scale_setting_widget.state.image_list_model = proxy_model
+        self.scale_setting_widget.image_viewer.state = self.scale_setting_widget.state
+
+        # self.current_image_viewer = self.scale_setting_widget.image_viewer  # TODO unify image viewer and change only the context around it
         # self.hide_thumb_gui()
         # self.image_list.entered.disconnect(self.show_thumbnail_gui)
         # self.image_list.view_left.disconnect(self.hide_thumb_gui)
@@ -1353,11 +1436,11 @@ class MAPHIS(QMainWindow):
         # self.label_editor.ui.photo_view.removeWidget(self.scale_setting_widget)
         # self.label_editor.ui.photo_view.insertWidget(0, self.label_editor.image_viewer)
 
-        self.current_image_viewer = self.label_editor.image_viewer
+        # self.current_image_viewer = self.label_editor.image_viewer
 
         # self.image_list.setItemDelegate(self.thumbnail_delegate)
 
-        self.label_editor.image_viewer.set_photo(self.state.current_photo, True)
+        # self.label_editor.image_viewer.set_photo(self.state.current_photo, True)
 
         # self.label_editor.image_viewer.show()
 
@@ -1366,6 +1449,9 @@ class MAPHIS(QMainWindow):
         #
         # self.ui.tabWidget.setTabText(0, "Label editor")
 
+    def setMaximumSize(self, arg__1: PySide6.QtCore.QSize) -> None:
+        super(MAPHIS, self).setMaximumSize(arg__1)
+
 
 # if __name__ == "__main__":
 #     app = QApplication([])
