@@ -35,7 +35,7 @@ class RegionProperty:
         self.value: Optional[Union[Value, Tuple[List[Any], Union[CompoundUnit, Unit]]]] = None  # either Value, or a tuple of list of values(float, int...) and a Unit
         self.prop_type: PropertyType = PropertyType.Scalar
         self.num_vals: int = 1
-        self.val_names: List[str] = []  # names of individual scalar values or matrices
+        self._val_names: List[str] = []  # names of individual scalar values or matrices
         self.col_names: List[str] = []  # in the case when self.prop_type = PropertyType.NDArray
         self.row_names: List[str] = []  # same here
         self.vector_viz: Optional[QPixmap] = None
@@ -46,6 +46,7 @@ class RegionProperty:
         self.prop_comp_key: str = ''
         self.local_key: str = ''
         self.settings: typing.Dict[str, typing.Dict[str, str]] = {}
+        self._up_to_date: bool = True
 
     @property
     def info(self) -> Optional[Info]:
@@ -61,9 +62,7 @@ class RegionProperty:
         return f'{self.prop_comp_key}.{self.local_key}'
 
     def __str__(self) -> str:
-        # return f'{self.info.name}: {self.format_value()} {self.unit}'
-        if len(self._str_rep) == 0:
-            self._update_str_rep()
+        self._update_str_rep()
         return self._str_rep
 
     def _update_str_rep(self):
@@ -106,6 +105,16 @@ class RegionProperty:
             return val_string[:-2]
         raise ValueError(f'Unsupported type of value: {type(self.value)}')
 
+    @property
+    def val_names(self) -> typing.List[str]:
+        if len(self._val_names) == 0:
+            return [f'value_{i}' for i in range(self.num_vals)]
+        return self._val_names
+
+    @val_names.setter
+    def val_names(self, val_names: typing.Sequence[str]):
+        self._val_names = list(val_names)
+
     @classmethod
     def from_dict(cls, dict_obj: Dict[str, Any]) -> 'RegionProperty':
         reg_prop = RegionProperty()
@@ -124,6 +133,7 @@ class RegionProperty:
         reg_prop.prop_comp_key = dict_obj['prop_comp_key']
         reg_prop.local_key = dict_obj['local_key']
         reg_prop.settings = dict_obj['settings']
+        reg_prop._up_to_date = dict_obj.get('up_to_date', True)
         info = Info()
         info.name = dict_obj['name']
         info.key = dict_obj['key']
@@ -140,6 +150,15 @@ class RegionProperty:
             return False
         return hash(self) == hash(other)
 
+    @property
+    def up_to_date(self) -> bool:
+        return self._up_to_date
+
+    @up_to_date.setter
+    def up_to_date(self, is_up_to_date: bool):
+        self._up_to_date = is_up_to_date
+        # TODO fire a signal
+
 
 class LabelImgType(IntEnum):
     Mask = 0,
@@ -171,6 +190,8 @@ class LabelImg(QObject):
     property_added = Signal(object, RegionProperty)
     property_removed = Signal(object, RegionProperty)
     property_updated = Signal(object, RegionProperty)
+    properties_need_recomputation = Signal(object)
+    all_properties_valid = Signal(object)
 
     def __init__(self, image_size: Tuple[int, int]):
         super(LabelImg, self).__init__(None)
@@ -217,6 +238,7 @@ class LabelImg(QObject):
             self.set_image(lbl_nd)
         self.is_segmented = bool(np.any(self._label_img > 0))
         self._dirty_flag = True
+        self._invalidate_measurements()
 
     @property
     def is_set(self) -> bool:
@@ -267,7 +289,8 @@ class LabelImg(QObject):
                             'description': prop.info.description,
                             'prop_comp_key': prop.prop_comp_key,
                             'local_key': prop.local_key,
-                            'settings': prop.settings
+                            'settings': prop.settings,
+                            'up_to_date': prop.up_to_date
                         }
                         for prop in prop_dict.values()
                     ]
@@ -307,6 +330,7 @@ class LabelImg(QObject):
                 raise ValueError(f'The dtype must be {self._label_img.dtype}, got {img.dtype}.')
         self._label_img = img
         self._dirty_flag = True
+        self._invalidate_measurements()
 
     def clone(self) -> 'LabelImg':
         lbl = LabelImg(self.size)
@@ -345,6 +369,8 @@ class LabelImg(QObject):
                 #label_prop.unit = prop.unit
                 #label_prop.prop_type = prop.prop_type
                 #label_prop.num_vals = prop.num_vals
+        if self.has_valid_measurements():
+            self.all_properties_valid.emit(self)
         self._dirty_flag = True
 
     def clear_region_props(self):
@@ -373,6 +399,8 @@ class LabelImg(QObject):
         self.prop_list.remove(reg_prop)
         self.set_dirty()
         self.property_removed.emit(self, reg_prop)
+        if self.has_valid_measurements():
+            self.all_properties_valid.emit(self)
 
     @property
     def label_hierarchy(self) -> LabelHierarchy:
@@ -402,6 +430,7 @@ class LabelImg(QObject):
 
     def set_dirty(self):
         self._dirty_flag = True
+        self._invalidate_measurements()
         self.timestamp = time.time()
 
     def _load_measurements(self):
@@ -428,6 +457,7 @@ class LabelImg(QObject):
         if self._label_img is None:
             self.reload()
         self._label_img = ndimage.rotate(self._label_img, 90 if ccw else -90, order=0, prefilter=False)
+        self.size = self._label_img.shape[::-1]
         self._dirty_flag = True
         if unload:
             self.unload()
@@ -458,7 +488,7 @@ class LabelImg(QObject):
         return level_img == label
 
     @property
-    def has_unsaved_changed(self) -> bool:
+    def has_unsaved_changes(self) -> bool:
         return self._dirty_flag
 
     @property
@@ -470,3 +500,17 @@ class LabelImg(QObject):
         top, left, bottom, right = np.min(rr), np.min(cc), np.max(rr), np.max(cc)
 
         return left, top, right - left + 1, bottom - top + 1
+
+    def _invalidate_measurements(self):
+        for region_label, region_props in self._region_props.items():
+            for region_prop, region_prop in region_props.items():
+                region_prop.up_to_date = False
+        # TODO fire a signal
+        self.properties_need_recomputation.emit(self)
+
+    def has_valid_measurements(self) -> bool:
+        for region_label, region_props in self._region_props.items():
+            for prop_key, region_prop in region_props.items():
+                if not region_prop.up_to_date:
+                    return False
+        return True
