@@ -1,7 +1,8 @@
 import typing
 
 from PySide6.QtWidgets import QWidget, QVBoxLayout, QGridLayout, QLabel, QGroupBox
-from common.user_param import Param, BoolParam
+
+from maphis.common.user_param import Param, BoolParam
 
 
 class ParamWidget:
