@@ -5,29 +5,27 @@ import typing
 from pathlib import Path
 from typing import Optional, List
 
-import numpy as np
 from PySide6.QtCore import Signal, QObject, QSize, QItemSelectionModel, QPoint, QModelIndex
 from PySide6.QtGui import Qt, QImage, QPixmap, QColor, QAction
 from PySide6.QtWidgets import QWidget, QToolButton, QGroupBox, QVBoxLayout, QGridLayout, QSizePolicy, QButtonGroup, \
     QAbstractButton, QRadioButton, QHBoxLayout, QSpacerItem, QLabel, QMenu
 
 from maphis.common.edit_command_executor import EditCommandExecutor
-from maphis.common.label_change import LabelChange, DoType, CommandEntry, CommandKind
-from maphis.common.label_hierarchy import Node
+from maphis.common.label_change import DoType, CommandEntry, CommandKind
+from maphis.common.label_hierarchy import Node, LabelHierarchy
 from maphis.common.label_tree_model import LabelTreeModel
 from maphis.common.label_tree_view import LabelTreeView
 from maphis.common.local_storage import Storage
-from maphis.common.photo import Photo, UpdateContext, UpdateEvent
-from maphis.common.photo_layer import PhotoLayer
+from maphis.common.photo import Photo
+from maphis.layers.photo_layer import PhotoLayer
 from maphis.common.state import State
 from maphis.common.tool import Tool
 from maphis.common.undo_manager import UndoManager
-from maphis.common.user_params import UserParamWidgetBinding, \
-    create_params_widget
-from maphis.common.visualization_layer import VisualizationLayer
+from maphis.common.user_params import UserParamWidgetBinding
+from maphis.layers.visualization_layer import VisualizationLayer
 from maphis.image_viewer import ImageViewer
 from maphis.label_editor.computation_widget import ComputationWidget
-from maphis.label_editor.label_layer import LabelLayer
+from maphis.layers.label_layer import LabelLayer
 from maphis.label_editor.label_level_switch import LabelLevelSwitch
 from maphis.label_editor.new_label_dialog import NewLabelDialog
 from maphis.label_editor.ui_label_editor import Ui_LabelEditor
@@ -195,13 +193,30 @@ class LabelEditor(QObject):
 
     def _setup_label_level_switch(self):
         self._label_switch = LabelLevelSwitch(self.state, self.widget)
-        self.state.label_hierarchy_changed.connect(self._label_switch.set_label_hierarchy)
+        # self.state.label_hierarchy_changed.connect(self._label_switch.set_label_hierarchy)
+        self.state.label_hierarchy_changed.connect(self.handle_label_hierarchy_changed)
         self._label_switch.approval_toggled.connect(self.toggle_approval_for)
 
         lay = QHBoxLayout()
         self.image_viewer.ui.controlBar.addWidget(self._label_switch)
         self.image_viewer.ui.controlBar.addSpacerItem(QSpacerItem(100, 0, QSizePolicy.Expanding, QSizePolicy.Fixed))
 
+    def handle_label_hierarchy_changed(self, lab_hier: LabelHierarchy):
+        print('reacting to label hierarchy change')
+        self._label_switch.set_label_hierarchy(lab_hier)
+        self.adjust_level_approval_switch_width()
+
+    def adjust_level_approval_switch_width(self):
+        width = 0
+        for idx in range(self.image_viewer.ui.controlBar.count()):
+            widget = self.image_viewer.ui.controlBar.itemAt(idx).widget()
+            if widget is None or isinstance(widget, LabelLevelSwitch):
+                continue
+            width += widget.width()
+        width += self.image_viewer.ui.controlBar.count() * self.image_viewer.ui.controlBar.layout().spacing()
+        width_to_set = int(0.95 * (self.image_viewer.width() - width))
+        self._label_switch.setMaximumWidth(width_to_set)
+
     def _setup_image_viewer(self):
         for i in range(self.ui.toolBar.count()):
             item = self.ui.toolBar.itemAt(i)
@@ -298,6 +313,7 @@ class LabelEditor(QObject):
         self.tool_box_layout.addWidget(self.tool_settings)
 
         self.tool_out_widget = QGroupBox()
+        self.tool_out_widget.setVisible(False)
         self.tool_box_layout.addWidget(self.tool_out_widget)
 
     def set_photo(self, photo: Optional[Photo], reset_view: bool=False):
@@ -352,6 +368,8 @@ class LabelEditor(QObject):
         print(f'{"activating" if checked else "deactivating"} tool id {self._tools_button_group.id(btn)}')
         tool_id = self._tools_button_group.id(btn)
         if checked:
+            if self._current_tool is not None:
+                self._current_tool.deactivate()
             for btn_ in self._tools_button_group.buttons():
                 if btn == btn_:
                     continue
@@ -361,24 +379,31 @@ class LabelEditor(QObject):
             if self.state.colormap is not None:
                 self._current_tool.color_map_changed(self.state.colormap)
             self.image_viewer.set_tool(self._current_tool)
+            self._current_tool.activate()
+            self._current_tool.switch_to_photo(self.state.current_photo)
             if self._current_tool.setting_widget is not None:
                 self.tool_settings.setVisible(True)
                 if self._current_tool_widget is not None:
                     self._current_tool_widget.setVisible(False)
                     self.tool_settings.layout().removeWidget(self._current_tool_widget)
-                    self._current_tool_widget.deleteLater()
                 self._current_tool_widget = self._current_tool.setting_widget
                 self.tool_settings.layout().addWidget(self._current_tool_widget)
                 self.tool_settings.setVisible(True)
                 self._current_tool_widget.setVisible(True)
             else:
                 self.tool_settings.setVisible(False)
+            self.image_viewer.photo_view.set_should_autoscroll(True)
+            self.image_viewer.photo_view.set_autoscroll_with_left_button_released(self._current_tool.should_auto_scroll_with_left_button_released())
+            self.image_viewer.photo_view.set_autoscroll_distance(self._current_tool.get_auto_scroll_distance())
         else:
+            self._current_tool.deactivate()
             self._current_tool = None
             self.image_viewer.set_tool(None)
             self._tool_param_widgets[tool_id].setVisible(False)
             self.tool_settings.setVisible(False)
             self.tool_out_widget.setVisible(False)
+            self.image_viewer.photo_view.set_should_autoscroll(False)
+            self.image_viewer.photo_view.set_autoscroll_with_left_button_released(False)
         self.tool_settings.update()
 
     def handle_label_changed(self, command: Optional[CommandEntry]):
@@ -461,7 +486,7 @@ class LabelEditor(QObject):
             label_node = lab_hier.nodes[label]
             label_level_to_visualize = label_level if len(label_node.children) > 0 else len(lab_hier.mask_names) - 1
             old_label_level = lab_hier.get_level(old_label)
-            if label_level_to_visualize != old_label_level:
+            if label_level_to_visualize != old_label_level or old_label == 0:
                 self.visualize_label_level(label_level_to_visualize)
         if self.state.redraw_canvas:
             self.label_layer.handle_primary_label_changed()
@@ -519,6 +544,7 @@ class LabelEditor(QObject):
 
         self.undo_manager.initialize(self.state.storage)
         self.image_viewer.set_storage(self.state.storage)
+        self.adjust_level_approval_switch_width()
 
     def handle_label_type_button_clicked(self, btn: QAbstractButton):
         # TODO This is only a temporary fix for Issue #23
@@ -536,6 +562,7 @@ class LabelEditor(QObject):
         self._label_switch.set_label_hierarchy(
             self.state.storage.get_label_hierarchy(self.state.current_label_name)
         )
+        self.adjust_level_approval_switch_width()
         lab_hier = self.state.storage.get_label_hierarchy(self.state.current_label_name)
 
         approved_level = self.state.current_photo.approved[self.state.current_label_name]
