@@ -86,11 +86,13 @@ class LabelLevelSwitch(QGroupBox):
             btn.setCheckable(True)
             #btn.setIcon(self.question_icon)
             btn.setIconSize(QSize(24, 24))
+            btn.setToolTip(name)
             # if i == 0:
             #     btn.setChecked(True)
             self.layout.addWidget(btn)
             self._label_buttons_group.addButton(btn, i)
             self._buttons.append(btn)
+        self.setMaximumWidth(self.maximumWidth())
         self.setEnabled(True)
         self.update()
 
@@ -115,3 +117,14 @@ class LabelLevelSwitch(QGroupBox):
         #    btn.setIcon(self.check_icon)
         #else:
         #    btn.setIcon(self.question_icon)
+    
+    def setMaximumWidth(self, maxw: int) -> None:
+        if len(self._buttons) > 0:
+            left, top, right, bottom = self.layout.getContentsMargins()
+            usable_width = maxw - left - right - ((2 * (len(self._buttons) - 1)) + 2) * self.layout.spacing()
+            width_per_item = (usable_width / len(self._buttons)) - 32
+            metrics = self._buttons[0].fontMetrics()
+            for btn, mask_name in zip(self._buttons, self._label_hierarchy.mask_names):
+                elided_text = metrics.elidedText(mask_name, Qt.TextElideMode.ElideRight, width_per_item)
+                btn.setText(elided_text)
+        super(LabelLevelSwitch, self).setMaximumWidth(maxw)
