@@ -1,4 +1,5 @@
 import dataclasses
+import itertools
 import re
 import typing
 from pathlib import Path
@@ -12,12 +13,14 @@ from PySide6.QtCore import QUrl
 from PySide6.QtWidgets import QMessageBox, QWidget, QPushButton, QVBoxLayout, QDialogButtonBox, \
     QDialog, QApplication, QScrollArea, QSizePolicy, QHBoxLayout, QFileDialog
 
+from maphis.common.units import Value, convert_value
 from maphis.common.common import Info
 from maphis.common.photo import Photo
 from maphis.common.plugin import GeneralAction, ActionContext
 from maphis.common.state import State
 from maphis.common.storage import Storage
-from maphis.common.utils import open_with_default_app
+from maphis.common.utils import open_with_default_app, let_user_save_path, attempt_to_save_with_retries, save_text, \
+    catch_exception_and_show_messagebox
 from maphis.plugins.profile_register.general.profiles import get_median_profile, merge_profiles
 from maphis.plugins.profile_register.general.tag_mapping_widget import TagMappingWidget
 from maphis.tags.tag_chooser import Tags
@@ -50,6 +53,13 @@ class SettingWidget:
     diagButtonBox: typing.Optional[QDialogButtonBox]
 
 
+@dataclasses.dataclass
+class ProfileFusionResult:
+    src_median_profile: np.ndarray
+    dst_median_profile: np.ndarray
+    aligned_profiles: np.ndarray
+
+
 class ProfileFusion(GeneralAction):
     """
     NAME: Profile fusion
@@ -67,7 +77,9 @@ class ProfileFusion(GeneralAction):
         if not self._check_for_profile_measurement(state):
             return
         self._state = state
-        self._execute(state, _)
+
+        self._setting_widget = self._build_settings_widget2(state, state.storage)
+        self._setting_widget.toplevel_dialog.open()
 
     def _get_median_profile_for(self, iteration_tag: str, group_tags: typing.Set[str], state: State) -> typing.Optional[np.ndarray]:
         tags_set = set(group_tags).union({iteration_tag})
@@ -94,42 +106,71 @@ class ProfileFusion(GeneralAction):
         return True
 
     def _execute(self, state: State, _: ActionContext):
-        self._setting_widget = self._build_settings_widget2(state, state.storage)
-        if self._setting_widget.toplevel_dialog.exec_() == QDialog.DialogCode.Accepted:
-            output_folder = Path(state.storage.location) / 'registered_profiles'
-            if not output_folder.exists():
-                output_folder.mkdir(exist_ok=True)
-            # TODO add an option to overwrite the existing results
-            # if self.chk_delete_existing.isChecked():
-            #     os.remove(output_folder / 'profiles.xlsx')
-            if (worksheet_path := output_folder / 'profiles.xlsx').exists():
-                wb = openpyxl.load_workbook(worksheet_path, read_only=False)
-            else:
-                wb = openpyxl.Workbook()
-                ws_med_prof = wb.active
-                ws_med_prof.title = 'Median profiles'
-                ws_med_prof.append(['ProfileID'] + [f'G_BP_{i}' for i in range(40)])
-
-                ws_aligned = wb.create_sheet('Aligned profiles')
-                ws_aligned.append(['ProfileID', 'AlignedTo'] + [f'G_BP_{i}' for i in range(40)])
-
-            ws_med_prof = wb['Median profiles']
-            ws_aligned = wb['Aligned profiles']
-
-            mappings = self._gather_mappings()
-            for mapping in mappings:
-                src, dsts = mapping
-                src_median, dst_median, aligned = self._register(src, dsts, state)
-                src_median_profile_id = ','.join(src)
-                dst_median_profile_id = '_'.join([','.join(dst_tags) for dst_tags in dsts])
-                ws_med_prof.append([src_median_profile_id] + [str(float(val)) for val in src_median])
-                ws_med_prof.append([dst_median_profile_id] + [str(float(val)) for val in dst_median])
-                ws_aligned.append([src_median_profile_id, dst_median_profile_id] + [str(float(val)) for val in aligned])
-            wb.save(worksheet_path)
-            wb.close()
-            self._show_success_info_dialog(worksheet_path)
+        output_folder = Path(state.storage.location) / 'registered_profiles'
+        if not output_folder.exists():
+            output_folder.mkdir(exist_ok=True)
+        # TODO add an option to overwrite the existing results
+        # if self.chk_delete_existing.isChecked():
+        #     os.remove(output_folder / 'profiles.xlsx')
+        if (worksheet_path := output_folder / 'profiles.xlsx').exists():
+            wb = openpyxl.load_workbook(worksheet_path, read_only=False)
         else:
-            pass
+            wb = openpyxl.Workbook()
+            ws_med_prof = wb.active
+            ws_med_prof.title = 'Median profiles'
+            ws_med_prof.append(['ProfileID', 'unit'] + [f'G_BP_{i}' for i in range(40)])
+
+            ws_aligned = wb.create_sheet('Aligned profiles')
+            ws_aligned.append(['ProfileID', 'AlignedTo', 'unit'] + [f'G_BP_{i}' for i in range(40)])
+
+        ws_med_prof = wb['Median profiles']
+        ws_aligned = wb['Aligned profiles']
+
+        mappings = self._gather_mappings()
+        for mapping in mappings:
+            src, dsts = mapping
+            fusion_result_mm, ignored_photos = self._register(src, dsts, state)
+            src_median_profile_id = ','.join(src)
+            dst_median_profile_id = '_'.join([','.join(dst_tags) for dst_tags in dsts])
+            if fusion_result_mm is not None:
+                ws_med_prof.append([src_median_profile_id, 'mm'] + [float(val) for val in fusion_result_mm.src_median_profile])
+                ws_med_prof.append([dst_median_profile_id, 'mm'] + [float(val) for val in fusion_result_mm.dst_median_profile])
+                ws_aligned.append([src_median_profile_id, dst_median_profile_id, 'mm'] + [float(val) for val in fusion_result_mm.aligned_profiles])
+            if len(ignored_photos) > 0:
+                QMessageBox.information(self._setting_widget.toplevel_dialog, 'Some profiles were ignored',
+                                               f'{len(ignored_photos)} profile(s) were ignored because they were measured in pixel units.',
+                                               QMessageBox.StandardButton.Ok)
+        self._attempt_to_save_results(wb, worksheet_path)
+
+    def _attempt_to_save_results(self, wb: openpyxl.Workbook, path: Path) -> bool:
+        results_saved_or_cancelled = False
+        while not results_saved_or_cancelled:
+            try:
+                wb.save(path)
+                wb.close()
+                results_saved_or_cancelled = True
+                self._show_success_info_dialog(path)
+                return True
+            except PermissionError:
+                path = self._show_error_and_ask_for_other_path(path)
+                if path is None:
+                    return False
+        return False
+
+    def _show_error_and_ask_for_other_path(self, old_path: Path) -> typing.Optional[Path]:
+        diag = QMessageBox(self._setting_widget.toplevel_dialog)
+        diag.setWindowTitle('Cannot save the results')
+        diag.setText(
+            f"The file {old_path} cannot be used for saving (maybe it's opened in another program).")
+        diag.addButton(QMessageBox.StandardButton.Cancel)
+        diag.addButton(QMessageBox.StandardButton.Open)
+        button: QPushButton = diag.button(QMessageBox.StandardButton.Open)
+        button.clicked.connect(diag.accept)
+        button.setText('Save as...')
+
+        if diag.exec_() == QMessageBox.DialogCode.Accepted:
+            return let_user_save_path(old_path.parent, 'Save as', 'single_file', ['xlsx'], self._setting_widget.toplevel_dialog)
+        return None
 
     def _gather_mappings(self) -> typing.List[typing.Tuple[Tags, GroupsOfTags]]:
         mappings: typing.List[typing.Tuple[Tags, GroupsOfTags]] = []
@@ -138,24 +179,55 @@ class ProfileFusion(GeneralAction):
             mappings.append(tag_mapping.get_mapping())
         return mappings
 
-    def _register(self, src: typing.List[str], dsts: typing.List[typing.List[str]], state: State):
-        src_profiles = self._gather_profiles(src, state)
-        src_median_profile = get_median_profile(src_profiles)
+    def _register(self, src: typing.List[str], dsts: typing.List[typing.List[str]], state: State) -> typing.Tuple[typing.Optional[ProfileFusionResult], typing.List[Photo]]:
+        src_profiles_mm, ignored_photos = self._gather_profiles(src, state)
+        ignored_photos: typing.Set[Photo] = set(ignored_photos)
 
-        dst_median_profiles: typing.List[np.ndarray] = []
+        if src_profiles_mm is None:
+            return None, list(ignored_photos)
+        src_median_profile_mm = get_median_profile(src_profiles_mm)
+
+        dst_median_profiles_mm: typing.List[np.ndarray] = []
         for dst in dsts:
-            dst_profiles = self._gather_profiles(dst, state)
-            dst_median_profiles.append(get_median_profile(dst_profiles))
-        dst_median_profile = get_median_profile(np.array(dst_median_profiles))
+            dst_profiles_mm, dst_ignored_photos = self._gather_profiles(dst, state)
+            ignored_photos.update(dst_ignored_photos)
+            if dst_profiles_mm is not None:
+                dst_median_profiles_mm.append(get_median_profile(dst_profiles_mm))
 
-        aligned = merge_profiles(src_median_profile, dst_median_profile, x_weight=0.5, y_weight=0.5)
+        if len(dst_median_profiles_mm) > 0:
+            dst_median_profile_mm = get_median_profile(np.array(dst_median_profiles_mm))
+        else:
+            dst_median_profile_mm = None
 
-        return src_median_profile, dst_median_profile, aligned
+        if dst_median_profile_mm is not None:
+            aligned_profiles_mm = merge_profiles(src_median_profile_mm, dst_median_profile_mm, x_weight=0.5, y_weight=0.5)
+            fusion_result_mm = ProfileFusionResult(src_median_profile_mm, dst_median_profile_mm, aligned_profiles_mm)
+        else:
+            fusion_result_mm = None
+        return fusion_result_mm, list(ignored_photos)
 
-    def _gather_profiles(self, tags: typing.List[str], state: State) -> np.ndarray:
+    def _gather_profiles(self, tags: typing.List[str], state: State) \
+            -> typing.Tuple[Optional[np.ndarray], typing.List[Photo]]:
         photos: typing.List[Photo] = state.storage.photos_satisfying_tags(set(tags))
-        profiles: typing.List[np.ndarray] = [np.array(photo['Labels'].region_props[BODY][PROFILE_KEY].value[0]) for photo in photos]
-        return np.array(profiles)
+
+        photos_with_scale: typing.List[Photo] = [photo for photo in photos if photo.image_scale is not None]
+        photos_no_scale: typing.List[Photo] = [photo for photo in photos if photo.image_scale is None]
+
+        # profiles_real_units: typing.List[np.ndarray] = [np.array(photo['Labels'].region_props[BODY][PROFILE_KEY].value[0]) for photo in photos_with_scale]
+
+        profiles_mm: typing.List[np.ndarray] = []
+        for photo in photos_with_scale:
+            profile_prop = photo['Labels'].region_props[BODY][PROFILE_KEY]
+            profile_mm = np.zeros((profile_prop.num_vals, ))
+            for idx, val in enumerate(profile_prop.value[0]):
+                value = Value(val, profile_prop.value[1])
+                value_mm = convert_value(value, state.units.units['mm'])
+                profile_mm[idx] = value_mm.value
+            profiles_mm.append(profile_mm)
+
+        profiles_mm: typing.Optional[np.ndarray] = np.array(profiles_mm) if len(profiles_mm) > 0 else None
+
+        return profiles_mm, photos_no_scale
 
     def _show_success_info_dialog(self, output_path: Path):
         diag_box = QMessageBox(QMessageBox.Icon.Information, "Profile fusion completed",
@@ -213,6 +285,7 @@ class ProfileFusion(GeneralAction):
             btnLoadConf=QPushButton('Load configuration...'),
             diagButtonBox=QDialogButtonBox(QDialogButtonBox.StandardButton.Apply | QDialogButtonBox.StandardButton.Cancel)
         )
+        sett_widget.toplevel_dialog.setWindowTitle('Profile fusion')
         sett_widget.toplevel_dialog.setLayout(QVBoxLayout())
         sett_widget.toplevel_dialog.layout().addWidget(sett_widget.scroll_area)
         sett_widget.toplevel_dialog.setMinimumHeight(600)
@@ -247,9 +320,10 @@ class ProfileFusion(GeneralAction):
 
         button_diag_box = sett_widget.diagButtonBox #QDialogButtonBox(QDialogButtonBox.StandardButton.Apply | QDialogButtonBox.StandardButton.Cancel)
         top_level_buttons_layout.addWidget(button_diag_box)
-        button_diag_box.button(QDialogButtonBox.StandardButton.Apply).clicked.connect(sett_widget.toplevel_dialog.accept)
+        # button_diag_box.button(QDialogButtonBox.StandardButton.Apply).clicked.connect(sett_widget.toplevel_dialog.accept)
+        button_diag_box.button(QDialogButtonBox.StandardButton.Apply).clicked.connect(lambda: self._execute(state, None))
         button_diag_box.button(QDialogButtonBox.StandardButton.Apply).setEnabled(False)
-        button_diag_box.button(QDialogButtonBox.StandardButton.Cancel).clicked.connect(sett_widget.toplevel_dialog.reject)
+        button_diag_box.button(QDialogButtonBox.StandardButton.Cancel).clicked.connect(sett_widget.toplevel_dialog.close)
 
         # self._setting_widget.diagButtonBox = button_diag_box
 
@@ -270,7 +344,12 @@ class ProfileFusion(GeneralAction):
             return
         load_path = Path(qurl.toLocalFile())
         self._save_recent_open_save_location(load_path.parent)
+        self._try_to_load_configuration(load_path=load_path)
 
+    @catch_exception_and_show_messagebox('Loading configuration failed',
+                                         'Could not load configuration from {load_path}',
+                                         QMessageBox.Icon.Critical, print_exception=True)
+    def _try_to_load_configuration(self, load_path: Path):
         with open(load_path, 'r') as f:
             str_mappings = [line.strip() for line in f.readlines()]
         valid_mappings: typing.List[typing.Tuple[Tags, GroupsOfTags]] = []
@@ -295,33 +374,45 @@ class ProfileFusion(GeneralAction):
     def _save_configuration(self):
         mappings = self._gather_mappings()
         if (recent_folder := self._get_recent_open_save_location()) is None:
-            recent_folder = QUrl()
+            recent_folder = Path(QUrl().toString())
         else:
-            recent_folder = QUrl.fromLocalFile(str(recent_folder))
-        result = QFileDialog.getSaveFileUrl(self._setting_widget.toplevel_dialog, 'Save configuration as...', dir=recent_folder)
-        qurl: QUrl = result[0]
-        if qurl.isEmpty():
+            recent_folder = Path(QUrl.fromLocalFile(str(recent_folder)).toString())
+        save_path = let_user_save_path(recent_folder, extensions=['txt'], parent=QApplication.activeWindow())
+        if save_path is None:
             return
-        save_path = Path(qurl.toLocalFile())
         self._save_recent_open_save_location(save_path.parent)
 
-        with open(save_path, 'w') as f:
-            for tags, groups_of_tags in mappings:
-                line = f'{dequote(str(tags))} {dequote(str(groups_of_tags))}\n'
-                f.write(line)
+        lines = []
+        for tags, groups_of_tags in mappings:
+            line = f'{dequote(str(tags))} {dequote(str(groups_of_tags))}\n'
+            lines.append(line)
+
+        config = '\n'.join(lines)
+
+        attempt_to_save_with_retries(save_text, 'Save as...', 'single_file',
+                                     ['txt'], QApplication.activeWindow(),
+                                     text=config, path=save_path)
 
     def _get_recent_open_save_location(self) -> typing.Optional[Path]:
-        with importlib.resources.path('maphis.plugins.profile_register.general', '.') as path:
-            if not (recent_path_file := path / 'recent_save_open_path.txt').exists():
-                return Path.home()
-            with open(recent_path_file, 'r') as f:
-                recent_path = Path(f.read())
-            return recent_path
+        try:
+            with importlib.resources.path('maphis.plugins.profile_register.general', '.') as path:
+                if not (recent_path_file := path / 'recent_save_open_path.txt').exists():
+                    return Path.home()
+                with open(recent_path_file, 'r') as f:
+                    recent_path = Path(f.read())
+                return recent_path
+        except Exception:
+            # TODO log it
+            return None
 
     def _save_recent_open_save_location(self, loc: Path):
-        with importlib.resources.path('maphis.plugins.profile_register.general', '.') as path:
-            with open(path / 'recent_save_open_path.txt', 'w') as f:
-                f.write(str(loc))
+        try:
+            with importlib.resources.path('maphis.plugins.profile_register.general', '.') as path:
+                with open(path / 'recent_save_open_path.txt', 'w') as f:
+                    f.write(str(loc))
+        except Exception:
+            # TODO log it
+            return
 
 
 def dequote(text: str) -> str:
